% TEMPSPOL: a MATLAB thermal model for deep subduction zones including major phase transformations
% A.M. Negredo, J.L. Valera and E. Carminati, Computers & Geosciences 30 (2004), 249-258

clear;

%%%%%%%%%%%%%%%%%%% INPUT PARAMETERS %%%%%%%%%%%%%%%%%%%%%%%%%
Dz=5000;		        % Vertical step (divisor of depth and L) (m)
K=3.2000; 	    	    	% Thermal conductivity (W/mK)
Cp=1.3E+3; 	    	    	% Specific heat (J/kg)
alpha=3.7E-5;          		% Thermal expansion coefficient(1/K or 1/?C)
Hp=0.00E-7; 	        	% Crustal radiactive heat production (W/m^3). Introduce 0 for oceanic subduction.
Lt=-9.0E4; 		        % Latent heat of ol-sp phase change (J/Kg). It must be negative for exothermic changes.
T0=1450.0; 		        % Temperature at the base of lithosphere (?C)
rhoc=2800.0; 		    	% Crustal density (Kg/m^3)
rhom=3400.0; 		    	% Reference mantle density (Kg/m^3)
drho_olsp=181.00;	    	% Density change during ol-sp transformation (Kg/m^3)
hc=0;     		        % Continental crustal thikness (m); use 0 for oceanic subduction
L=95000; 		        % Lithospheric thickness (m)
depth=670000; 		    	% Depth of the base of the model domain (m)
g=9.8100; 	    	    	% Gravitational acceleration (m/s^2)
dip=60.000; 		    	% Subduction angle (grad)
dip=dip*pi/180;			% Change to radians
Vs=-10.0;  		        % Subduction velocity (Vs<0 means subduction to the left) (cm/yr)
Vs=Vs*1e-2/31536000;		% Change to m/sec
age=120.0; 		        % Age of subducting lithosphere (Myr)
age=age*1e6*31536000;   	% Change to sec.
taudat=0e7; 		    	% Shear stress (Pa)
tend=6.4;        		% Total time of evolution (Myr)
tend=tend*1e6*31536000;		% Change to sec.
x1=500000;	        	% x of the hinge line (m)
xend=700000; 		    	% Total length (m)
%%%%%%%%%%%%%%%%%%% End of Input Parameters %%%%%%%%%%%%%%%%%%

% answ=input ('Do you want to save the parameters of your model? (''Y''/''N'') ');
% if answ=='Y' | answ=='y'
% 	save model.dat -ascii;
% 	disp ('Model parameters saved in model.dat ascii-file');
% else
% 	disp ('OK. Model parameter will not be saved');
% end
save model.dat -ascii;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Other Parameters %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Dt=Dz/2/(abs(Vs)*sin(dip));     % time interval is forced to be the time needed to advance dz/2
Dx=Dz/tan(dip);			% Create the grid

disp('tend (Myr)');
disp(tend/1e6/31536000);
disp('Total number of iterations');
disp(tend/Dt);
% iter=input('itersave?');	% t_max is 222 here ~ that is there is
iter=223;			% NO saving for the given parameter set
itersave=iter;
P=round(tend/Dt); 	    	% number of time iterations
J=depth/Dz+1;           	% number of nodes in the vertical direction
I=round(xend/Dx)+1; 		% number of nodes in the horizontal direction
%%%%%%%%%%%%%%%%%%%%%%%%%%%% End of other Parameters %%%%%%%%%%%%%%%%%%%%%%%%%%

disp('P'); disp(P);
disp('J'); disp(J);
disp('I'); disp(I);
disp('These are the number of iterations of time (P), and in the horizontal');
disp('	 scale (I) and vertical scale (J).');
disp('Note that if J is not an integer, the top of the model will not be the');
disp('	 surface');
% answ=input('Do you agree? (''y''/''n'') ');
% if answ=='n' | answ=='N'
%  	error('You disagree with this values. The program is stopped');
% elseif answ=='y' | answ=='Y'
% 	t=0;
t=0;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% MODEL %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%% Calculate the initial distribution of temperature, density, velocity, shear heating rate and the
%%%% 	fraction of spinel (beta) and its derivatives

init_temp;
Valdelta_rho=rho-(ones(I,1)*rho(I,:));
ValT=T;
Valbeta=beta;
time=0;
init_velo;          		% Define velocity distribution
init_shearheat;     		% Defining shear heating rate distribution
if hc>0 
	calc_contrho; 		% calculates continental crust density and heat production if we have continental subduction
end

%%%% End of Calculating the initial distributions

%%%%%%%%%%%%%%%%%%%%%%%%% CALCULATING THE TIME EVOLUTION %%%%%%%%%%%%%%%%%%%%%%
for t=1:P
	disp (t)
	Dt1=Dt/2;

%%% Calculation of coefficients for the first substep
	for j=2:J-1
%% Thomas algorithm 
		w(1,j)=1;
		gg(1,j)=0;
		for i=2:I-1
			coeffx  
      			w(i,j)=-C/(B+A*w(i-1,j));
      			gg(i,j)=(D-A*gg(i-1,j))/(B+A*w(i-1,j));
    		end
		T(I,j)=gg(I-1,j)/(1-w(I-1,j));  % Value of T for the last column
		for i=I-1:-1:1
 			T(i,j)=w(i,j)*T(i+1,j)+gg(i,j);	
		end
%% End of Thomas algorithm
  	end
% initial values remain for the first and last row (j=1 and j=J)

	time=time+Dt1;

%%% Calculation of coefficients for the second substep
	for i=2:I-1
%% Thomas algorithm 
		w(i,1)=1;
    		gg(i,1)=qb(i)/K*Dz;
		for j=2:J-1
			coeffz
			w(i,j)=-C/(B+A*w(i,j-1));
      			gg(i,j)=(D-A*gg(i,j-1))/(B+A*w(i,j-1));
    		end
    		T(i,J)=0;  % Last value of T (at surface)
		for j=J-1:-1:1
			T(i,j)=w(i,j)*T(i,j+1)+gg(i,j);
		end
%% End of Thomas algorithm
	end
  
	time=time+Dt1;
	velo;

	if taudat>0     
		shearheat;      % calculates distribution of shear stress
	end
       	calc_beta;	% calculates beta and its derivatives, density and radiogenic heat production 
	if hc>0 
		calc_contrho; 	% calculates continental crust density and heat production if we have continental subduction
	end
%%%% Assign values to the first and last column
	T(1,:)=T(2,:);
 	T(I,:)=T(I-1,:);
% the temperature at the base of the subducting plate is maintained at T0
	jlit=round(za/Dz+1);
	ix0=round(x0/Dx+1);
	T(ix0:I,jlit)=T0;
%%%% Saving results every 'iter' iterations
	if t==iter
		disp('************************ saving *******************************')
		iter=iter + itersave;
		ValT=[ValT T];	
                delta_rho=rho-(ones(I,1)*rho(I,:));  %calculation of the lateral anomaly of density respect to the rightmost boundary
    		Valdelta_rho=[Valdelta_rho delta_rho];
    		Valbeta=[Valbeta beta];
  	end

end

ValT=[ValT T];			                % Storage of the last T-distribution	
delta_rho=rho-(ones(I,1)*rho(I,:));
Valdelta_rho=[Valdelta_rho delta_rho];  	% Storage of the last delta_rho-distribution
Valbeta=[Valbeta beta];		            	% Storage of the last beta-distribution
Valvelo=[vx vz];	                    	% Saving only the last V-distribution	

%%%%%%%%%%%%%%%%%%%%%%%%%%%% END OF TIME EVOLUTION %%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%% Saving instructions %%%%%
disp('End Time ');	
disp(time/1e6/31536000);
save T_file.m ValT -ascii;	                % Saving the Temperature distribution
save delta_rho_file.m Valdelta_rho -ascii;	% Saving the lateral anomaly of density 
save V_file.m Valvelo -ascii;	            % Saving the final Velocity distribution
save beta_file.m Valbeta -ascii;            % Saving the Beta distribution

for j=1:J
	z(j)=(j-1)*Dz;
end

for i=1:I
	x(i)=(i-1)*Dx;
end

arrange;			% Transforms x and z vectors into matrices
z=z-depth;

%
%	else
%  		error('This input is wrong. The program is stopped');
% end

clear answ w gg A B C D x0 x2 x3 x3old x4 x4old z3x4old answ c cold cx cz i index ix3old;
clear ix4 j jx3 jx4old rx rz t vv xpp3 Dt1 ixo ix1c jcrust jlit x1c x3old;
